@@ -412,7 +412,7 @@ draw_texture_square() ->
                    1.0:?F32,0.0:?F32, 1.0:?F32,0.0:?F32,
                    1.0:?F32,1.0:?F32, 1.0:?F32,1.0:?F32,
                    0.0:?F32,1.0:?F32, 0.0:?F32,1.0:?F32>>,
-    wings_vbo:draw(fun() -> gl:drawArrays(?GL_QUADS, 0, 4) end, VertexUvQ, [vertex2d, uv]).
+    wings_vbo:draw(fun(_) -> gl:drawArrays(?GL_QUADS, 0, 4) end, VertexUvQ, [vertex2d, uv]).
 
 fill_bg_tex(#sh_conf{fbo_w=Prev}) ->
     gl:drawBuffer(?GL_COLOR_ATTACHMENT1_EXT),
@@ -903,8 +903,8 @@ pass({auv_faces,[_]},_) ->
 	    R = fun(#fs{vs=Vs}) ->
 			wings_gl:drawElements(?GL_TRIANGLES,length(Vs),?GL_UNSIGNED_INT,Vs)
 		end,
-	    All = fun() -> foreach(fun(#chart{fs=Fs}) -> foreach(R, Fs) end, Charts) end,
-            EnableVbo(All)
+	    All = fun(_) -> foreach(fun(#chart{fs=Fs}) -> foreach(R, Fs) end, Charts) end,
+            EnableVbo(All, #{})
     end;
 pass({auv_faces, _},Sh) ->
     pass({auv_faces,?OPT_FACES},Sh);
@@ -951,7 +951,7 @@ shader_pass({value,#sh{id=Id, args=Args, tex_units=TexUnits}},
                                          end,
                                      foreach(R,Fas)
                              end,
-                        EnableVbo(fun() -> foreach(DC, Charts) end)
+                        EnableVbo(fun(_) -> foreach(DC, Charts) end, #{})
                 end
             catch throw:What ->
                     io:format("AUV: ERROR ~s ~n",[What]);
@@ -987,10 +987,10 @@ send_texture([],_) -> false.
 
 
 shader_uniforms([{uniform,color,Name,_,_}|As],[Val|Opts],Conf) ->
-    wings_gl:set_uloc(Conf#sh_conf.prog,Name,Val),
+    wings_gl:set_uloc(Conf#sh_conf.prog, Name,Val),
     shader_uniforms(As,Opts,Conf);
 shader_uniforms([{uniform,float,Name,_,_}|As],[Val|Opts],Conf) ->
-    wings_gl:set_uloc(Conf#sh_conf.prog,Name, Val),
+    wings_gl:set_uloc(Conf#sh_conf.prog, Name, Val),
     shader_uniforms(As,Opts,Conf);
 shader_uniforms([{uniform,menu,Name,_,_}|As],[Vals|Opts],Conf) 
   when is_list(Vals) ->
