@@ -144,7 +144,8 @@ create_window(Action, Name, Id, #st{shapes=Shs}=St) ->
     Canvas = wings_gl:window(Frame, Context, true, true),
     Props = [{display_data,Name}|wings_view:initial_properties()++Ps],
     wings_wm:toplevel(Name, Canvas, Props, Op),
-    wings_wm:send(Name, {init,{Action,We}}).
+    wings_wm:send(Name, {init,{Action,We}}),
+    Frame.
 
 auv_event({init,Op}, St) ->
     wings:init_opengl(St),
@@ -201,7 +202,14 @@ init_show_maps(Charts0, Fs, #we{name=WeName,id=Id}, GeomSt0) ->
 	    wings_wm:send(EditWin, {add_faces,Fs,GeomSt}),
 	    wings_wm:send(geom, {new_state,GeomSt});
 	false ->
-	    create_window({edit,Fs}, EditWin, Id, GeomSt),
+	    %% we are going to ensure to open the AutoUV window in the same
+	    %% display as the segment window was, since it can be in another
+	    %% than the one in which the main window is.
+	    SegWin = wings_wm:this_win(),
+	    {X0,Y0} = wxWindow:getPosition(SegWin),
+	    Pos = wxWindow:clientToScreen(SegWin,X0,Y0),
+	    Win = create_window({edit,Fs}, EditWin, Id, GeomSt),
+	    wxWindow:move(Win,Pos),
 	    wings_wm:send(geom, {new_state,GeomSt})
     end,
     GeomSt.
@@ -625,7 +633,7 @@ handle_event_3({action,{auv,{draw_options,Opt}}}, #st{bb=Uvs}=St) ->
     Tx = ?SLOW(auv_texture:get_texture(St, Opt)),
     case MatName0 of 
 	none -> 
-	    wings_image:update(Image, Tx),
+	    ok = wings_image:update(Image, Tx),
 	    put({?MODULE,show_background}, true),
 	    get_event(St);
 	_ ->
@@ -665,10 +673,38 @@ handle_event_3({action,{auv,Cmd}}, St) ->
 handle_event_3({action,{select,show_all}}, #st{bb=#uvstate{st=GeomSt,id=Id}}) ->
     wings_wm:send({autouv,Id}, {add_faces,object,GeomSt}),
     keep;
+handle_event_3({action,{select,oriented_faces}}, St0) ->
+    Connected = wings_pref:get_value(similar_normals_connected,false),
+    {Save,Angle} = case wings_pref:get_value(similar_normals_angle,{false,1.0E-3}) of
+		       {true,A} -> {true,A};
+		       {false,_} -> {false,1.0E-3}
+		   end,
+    handle_event_3({action,{select,{oriented_faces,[Angle,Connected,Save]}}}, St0);
+handle_event_3({action,{select,similar_area}}, St0) ->
+    handle_event_3({action,{select,{similar_area,[0.001]}}}, St0);
+handle_event_3({action,{select,similar_material}}, St0) ->
+    Connected = wings_pref:get_value(similar_materials_connected, false),
+    Mode = wings_pref:get_value(similar_materials, material),
+    handle_event_3({action,{select,{similar_material,[Connected,Mode]}}}, St0);
+handle_event_3({action,{select,{ssels,sel_groups_win}}}, _) ->
+    keep;
+handle_event_3({action,{select,deselect_previous}=Command}, St0) ->
+    case wpc_deselect_previous:command(Command, St0) of
+	{save_state,St} -> ok;
+	_ -> St = St0
+    end,
+    new_state(St);
 handle_event_3({action,{select,Command}}, St0) ->
     case wings_sel_cmd:command(Command, St0) of
 	{save_state,St} -> ok;
-	#st{}=St -> ok
+	#st{}=St -> ok;
+	_ ->
+	    %% That's avoid crash if any select option has a input dialog when
+	    %% usually the returned value returned at the first time is 'keep'.
+	    %% Also, for commands with preview dialog, the new state will not
+	    %% be properly updated since somehow the local St seems to be messed
+	    %% in {current_state,geom_display_lists,GeomSt} event handle
+	    St = St0
     end,
     new_state(St);
 handle_event_3({action,{edit,repeat}}, St) ->
@@ -1884,7 +1920,7 @@ draw_background(#st{bb=#uvstate{matname=MatName,st=St,bg_img=Image}}) ->
     gl:translatef(0.0, 0.0, -0.5),
     Bin = << <<V:?F32,20.0:?F32, V:?F32,-20:?F32, 20.0:?F32,V:?F32, -20.0:?F32,V:?F32>>
              || V <- lists:seq(-20,20) >>,
-    wings_vbo:draw(fun() -> gl:drawArrays(?GL_LINES, 0, 4*(20+20+1)) end, Bin, [vertex2d]),
+    wings_vbo:draw(fun(_) -> gl:drawArrays(?GL_LINES, 0, 4*(20+20+1)) end, Bin, [vertex2d]),
     gl:lineWidth(3.0),
     gl:color3f(0.0, 0.0, 1.0),
     gl:recti(0, 0, 1, 1),
@@ -1908,7 +1944,7 @@ draw_background(#st{bb=#uvstate{matname=MatName,st=St,bg_img=Image}}) ->
     end,
     Q = [{0.0, 0.0},{0.0, 0.0, -0.99999}, {1.0, 0.0},{1.0, 0.0, -0.99999},
          {1.0, 1.0},{1.0, 1.0, -0.99999}, {0.0, 1.0},{0.0, 1.0, -0.99999}],
-    wings_vbo:draw(fun() -> gl:drawArrays(?GL_QUADS, 0, 4) end, Q, [uv, vertex]),
+    wings_vbo:draw(fun(_) -> gl:drawArrays(?GL_QUADS, 0, 4) end, Q, [uv, vertex]),
 
     gl:disable(?GL_TEXTURE_2D),
 
