@@ -100,7 +100,7 @@ export_1(Filename, Contents0, Attr) ->
     Dir = filename:dirname(Filename),
     FileType = proplists:get_value(file_type, Attr, glb),
     ImageDir = case FileType of
-                   glb -> filename:basedir(user_cache, "wings3d");
+                   glb -> wings_u:basedir(user_cache);
                    gltf -> Dir
                end,
     ok = filelib:ensure_dir(filename:join(ImageDir, "dummy")),
@@ -175,26 +175,30 @@ exp_object(#e3d_object{name=Name, obj=WMesh}, {Ns,GLTF0}) ->
 
 exp_mesh(WMesh0, Name, GLTF0) ->
     WMesh1 = e3d_mesh:vertex_normals(WMesh0),
-    #e3d_mesh{vs=Vs,ns=Ns,tx=Tx} = WMesh1,
+    #e3d_mesh{vs=Vs,ns=Ns,tx=Tx,vc=Vc} = WMesh1,
     WMesh = WMesh1#e3d_mesh{vs=array:from_list(Vs),
                             ns=array:from_list(Ns),
-                            tx=array:from_list(Tx, {0.0,0.0})},
+                            tx=array:from_list(Tx, {0.0,0.0}),
+                            vc=array:from_list(Vc)},
     FacesByMaterial = segment_by_material(WMesh),
     {Prims,GLTF} = exp_mesh_1(FacesByMaterial, WMesh, GLTF0, [], #{}),
     {#{name=>Name, primitives=>Prims},GLTF}.
 
 exp_mesh_1([{[Mat|_], Fs}|MatFs], WMesh, GLTF0, Ps, S0) ->
     {MId, GLTF} = material_id(Mat, GLTF0),
-    {Inds, S} = case array:size(WMesh#e3d_mesh.tx) of
-                    0 -> exp_faces(Fs, WMesh, fun exp_face_n/3, [], S0);
-                    _ -> exp_faces(Fs, WMesh, fun exp_face_tx/3, [], S0)
+    {Inds, S} = case {array:size(WMesh#e3d_mesh.tx),
+		      array:size(WMesh#e3d_mesh.vc)} of
+		    {0,0} -> exp_faces(Fs, WMesh, fun exp_face_n/3, [], S0);
+		    {_,0} -> exp_faces(Fs, WMesh, fun exp_face_tx/3, [], S0);
+		    {0,_} -> exp_faces(Fs, WMesh, fun exp_face_vc/3, [], S0);
+		    {_,_} -> exp_faces(Fs, WMesh, fun exp_face_tx_vc/3, [], S0)
                 end,
     P = #{material=> MId, indices=>length(Fs)*3},
     exp_mesh_1(MatFs, WMesh, GLTF, [{P, Inds}|Ps], S);
-exp_mesh_1([], #e3d_mesh{tx=Tx, vs=Vs}, GLTF0, Ps0, S) ->
+exp_mesh_1([], #e3d_mesh{tx=Tx, vc=Vc, vs=Vs}, GLTF0, Ps0, S) ->
     {Ps, GLTF1} = exp_add_index(lists:reverse(Ps0), GLTF0),
     MinMax = e3d_bv:box(array:to_list(Vs)),
-    {Attr, GLTF} = exp_add_mesh_data(MinMax, array:size(Tx) =:= 0, S, GLTF1),
+    {Attr, GLTF} = exp_add_mesh_data(MinMax, array:size(Tx) =:= 0, array:size(Vc) =:= 0, S, GLTF1),
     {[P#{attributes=>Attr} || P <- Ps], GLTF}.
 
 exp_add_index(Ps0, GLTF0) ->
@@ -208,13 +212,15 @@ exp_add_index(Ps0, GLTF0) ->
                            {P#{indices:=AId}, GLTF}
                    end, GLTF1, Ps1).
 
-exp_add_mesh_data({Min, Max}, UseTx, S, GLTF0) ->
+exp_add_mesh_data({Min, Max}, NoTx, NoVc, S, GLTF0) ->
     VsData = lists:sort(maps:values(S)),
     N = maps:size(S),
     Bin = << <<Bin/binary>> || {_, Bin} <- VsData>>,
-    Stride = case UseTx of
-                 true -> 6*4;
-                 false -> 6*4+2*4
+    Stride = case {NoTx,NoVc} of
+		 {true,true} -> 6*4;
+		 {false,true} -> 6*4+2*4;
+		 {true,false} -> 6*4+4*4;
+		 {false,false} -> 6*4+2*4+4*4
              end,
 
     {BVId, GLTF1} = exp_add(#{buffer=>Bin, byteStride=> Stride}, bufferViews, GLTF0),
@@ -223,13 +229,23 @@ exp_add_mesh_data({Min, Max}, UseTx, S, GLTF0) ->
                            accessors, GLTF1),
     {NsA, GLTF3} = exp_add(exp_make_acc(BVId, N, ?GL_FLOAT, <<"VEC3">>, 3*4),
                            accessors, GLTF2),
-    case UseTx of
-        true ->
+    case {NoTx,NoVc} of
+	{true,true} ->
             {#{'POSITION'=>VsA, 'NORMAL'=> NsA}, GLTF3};
-        false ->
+	{false,true} ->
             {TxA, GLTF4} = exp_add(exp_make_acc(BVId, N, ?GL_FLOAT, <<"VEC2">>, 6*4),
                                    accessors, GLTF3),
-            {#{'POSITION'=>VsA, 'NORMAL'=> NsA, 'TEXCOORD_0' => TxA}, GLTF4}
+            {#{'POSITION'=>VsA, 'NORMAL'=> NsA, 'TEXCOORD_0' => TxA}, GLTF4};
+	{true,false} ->
+	    {VcA, GLTF4} = exp_add(exp_make_acc(BVId, N, ?GL_FLOAT, <<"VEC4">>, 6*4),
+				   accessors, GLTF3),
+	    {#{'POSITION'=>VsA, 'NORMAL'=> NsA, 'COLOR_0' => VcA}, GLTF4};
+	{false,false} ->
+	    {TxA, GLTF4} = exp_add(exp_make_acc(BVId, N, ?GL_FLOAT, <<"VEC2">>, 6*4),
+				   accessors, GLTF3),
+	    {VcA, GLTF5} = exp_add(exp_make_acc(BVId, N, ?GL_FLOAT, <<"VEC4">>, 6*4+2*4),
+				   accessors, GLTF4),
+	    {#{'POSITION'=>VsA, 'NORMAL'=> NsA, 'TEXCOORD_0' => TxA, 'COLOR_0' => VcA}, GLTF5}
     end.
 
 exp_faces([Face|Fs], WMesh, Fun, Inds, S0) ->
@@ -248,11 +264,29 @@ exp_face_n(#e3d_face{vs=[V1,V2,V3], ns=[N1,N2,N3]}, #e3d_mesh{vs=Vs,ns=Ns}, S0)
 exp_face_tx(#e3d_face{vs=[V1,V2,V3], ns=[N1,N2,N3], tx=FTx},
             #e3d_mesh{vs=Vs,ns=Ns,tx=Tx}, S0) ->
     [T1,T2,T3] = fix_tx(FTx),
-    {F1,S1} = exp_data(V1,N1,T1,Vs,Ns,Tx,S0),
-    {F2,S2} = exp_data(V2,N2,T2,Vs,Ns,Tx,S1),
-    {F3,S3} = exp_data(V3,N3,T3,Vs,Ns,Tx,S2),
+    {F1,S1} = exp_data(V1,N1,T1,Vs,Ns,Tx,none,S0),
+    {F2,S2} = exp_data(V2,N2,T2,Vs,Ns,Tx,none,S1),
+    {F3,S3} = exp_data(V3,N3,T3,Vs,Ns,Tx,none,S2),
     {[F3,F2,F1], S3}.
 
+exp_face_vc(#e3d_face{vs=[V1,V2,V3], ns=[N1,N2,N3], vc=FVc},
+	    #e3d_mesh{vs=Vs,ns=Ns,vc=Vc}, S0) ->
+    [VC1,VC2,VC3] = fix_tx(FVc),
+    {F1,S1} = exp_data(V1,N1,VC1,Vs,Ns,none,Vc,S0),
+    {F2,S2} = exp_data(V2,N2,VC2,Vs,Ns,none,Vc,S1),
+    {F3,S3} = exp_data(V3,N3,VC3,Vs,Ns,none,Vc,S2),
+    {[F3,F2,F1], S3}.
+
+exp_face_tx_vc(#e3d_face{vs=[V1,V2,V3], ns=[N1,N2,N3], tx=FTx, vc=FVc},
+	    #e3d_mesh{vs=Vs,ns=Ns,tx=Tx,vc=Vc}, S0) ->
+    [T1,T2,T3] = fix_tx(FTx),
+    [VC1,VC2,VC3] = fix_tx(FVc),
+    {F1,S1} = exp_data(V1,N1,{T1,VC1},Vs,Ns,Tx,Vc,S0),
+    {F2,S2} = exp_data(V2,N2,{T2,VC2},Vs,Ns,Tx,Vc,S1),
+    {F3,S3} = exp_data(V3,N3,{T3,VC3},Vs,Ns,Tx,Vc,S2),
+    {[F3,F2,F1], S3}.
+
+
 exp_data(V,N,Vs,Ns,S0) ->
     Key = {V,N},
     case maps:get(Key, S0, undefined) of
@@ -266,7 +300,7 @@ exp_data(V,N,Vs,Ns,S0) ->
             {Id, S0#{Key=>{Id, Bin}}}
     end.
 
-exp_data(V,N,Uv,Vs,Ns,Tx,S0) ->
+exp_data(V,N,Uv,Vs,Ns,Tx,none,S0) ->
     Key = {V,N,Uv},
     case maps:get(Key, S0, undefined) of
         {Id, _} -> {Id, S0};
@@ -276,11 +310,50 @@ exp_data(V,N,Uv,Vs,Ns,Tx,S0) ->
             {NX,NY,NZ} = array:get(N, Ns),
             {XU,XV} = array:get(Uv, Tx),
             Bin = << X:?F32L, Y:?F32L, Z:?F32L,
-                    NX:?F32L,NY:?F32L,NZ:?F32L,
-                    XU:?F32L,(1.0-XV):?F32L>>,
+                     NX:?F32L,NY:?F32L,NZ:?F32L,
+                     XU:?F32L,(1.0-XV):?F32L>>,
             {Id, S0#{Key=>{Id, Bin}}}
+    end;
+exp_data(V,N,Cl,Vs,Ns,none,Vc,S0) ->
+    Key = {V,N,Cl},
+    case maps:get(Key, S0, undefined) of
+	{Id, _} -> {Id, S0};
+	undefined ->
+	    Id = maps:size(S0),
+	    {X,Y,Z} = array:get(V, Vs),
+	    {NX,NY,NZ} = array:get(N, Ns),
+	    {R,G,B,A} =
+	    	case array:get(Cl, Vc) of
+		    {R0,G0,B0} -> {R0,G0,B0,1.0};
+		    RGBA -> RGBA
+		end,
+	    Bin = << X:?F32L, Y:?F32L, Z:?F32L,
+		     NX:?F32L,NY:?F32L,NZ:?F32L,
+		     R:?F32L,G:?F32L,B:?F32L,A:?F32L>>,
+	    {Id, S0#{Key=>{Id, Bin}}}
+    end;
+exp_data(V,N,{Uv,Cl},Vs,Ns,Tx,Vc,S0) ->
+    Key = {V,N,Uv},
+    case maps:get(Key, S0, undefined) of
+	{Id, _} -> {Id, S0};
+	undefined ->
+	    Id = maps:size(S0),
+	    {X,Y,Z} = array:get(V, Vs),
+	    {NX,NY,NZ} = array:get(N, Ns),
+	    {XU,XV} = array:get(Uv, Tx),
+	    {R,G,B,A} =
+	    case array:get(Cl, Vc) of
+		{R0,G0,B0} -> {R0,G0,B0,1.0};
+		RGBA -> RGBA
+	    end,
+	    Bin = << X:?F32L, Y:?F32L, Z:?F32L,
+		     NX:?F32L,NY:?F32L,NZ:?F32L,
+		     XU:?F32L,(1.0-XV):?F32L,
+		     R:?F32L,G:?F32L,B:?F32L,A:?F32L>>,
+	    {Id, S0#{Key=>{Id, Bin}}}
     end.
 
+
 exp_setup_buffers(#{bufferViews:=BVs0} = GLTF0) ->
     {Bin, BVs} = exp_setup_buffer(lists:reverse(BVs0), <<>>, []),
     {0, GLTF} = exp_add(#{byteLength=>byte_size(Bin)}, buffers, GLTF0),
@@ -303,20 +376,19 @@ exp_make_materials([Used|UMs], WMats, Type, GLTF0) ->
     WMat = proplists:get_value(Used,WMats),
     GL   = proplists:get_value(opengl, WMat),
     {DR,DG,DB,DA} = proplists:get_value(diffuse, GL),
-    {SR,SG,SB,_} = proplists:get_value(specular, GL),
-    Shin = proplists:get_value(shininess, GL),
+    Metallic = proplists:get_value(metallic, GL, 0.1),
+    Roughness = proplists:get_value(roughness, GL, 0.8),
     {ER,EG,EB,_} = proplists:get_value(emission, GL),
     Maps = proplists:get_value(maps, WMat, []),
     %% [io:format("Map: ~p ~p~n", [T, I#e3d_image{image= <<>>}]) || {T,I} <- Maps],
 
     Base0 = #{baseColorFactor=> [DR,DG,DB,DA],
-              metallicFactor => min(1.0, e3d_vec:dot(e3d_vec:norm({DR,DG,DB}),
-                                                     e3d_vec:norm({SR,SG,SB}))),
-              roughnessFactor=> max(0.0,min(1.0 - Shin, 1.0))},
+              metallicFactor => Metallic,
+              roughnessFactor=> Roughness},
     DiffMap = proplists:get_value(diffuse, Maps),
 
     {Base1,GLTF1} = exp_add_image(DiffMap, Base0, baseColorTexture, Type, GLTF0),
-    {Base, GLTF2} = exp_add_image(proplists:get_value(metallic, Maps),
+    {Base, GLTF2} = exp_add_image(merge_met_roughness(Maps),
                                   Base1, metallicRoughnessTexture, Type, GLTF1),
 
     Mat0 = #{pbrMetallicRoughness => Base,
@@ -344,6 +416,35 @@ exp_make_materials([Used|UMs], WMats, Type, GLTF0) ->
 exp_make_materials([], _, _, GLTF) ->
     GLTF.
 
+merge_met_roughness(Maps) ->
+    {FN, E3d} =
+        case {proplists:get_value(metallic, Maps),
+              proplists:get_value(roughness, Maps)} of
+            {undefined, undefined}=Undef -> Undef;
+            {#e3d_image{filename=File}=Map, undefined} ->
+                {File, e3d_image:expand_channel(b, e3d_image:convert(Map, g8, 1))};
+            {undefined, #e3d_image{filename=File}=Map} ->
+                {File, e3d_image:expand_channel(g, e3d_image:convert(Map, g8, 1))};
+            {#e3d_image{width=W, height=H}=Met, #e3d_image{width=W, height=H}=Rough} ->
+                RGBA0 = e3d_image:expand_channel(b, e3d_image:convert(Met, g8, 1)),
+                RGBA1 = e3d_image:replace_channel(g, e3d_image:convert(Rough, g8, 1), RGBA0),
+                {Met#e3d_image.filename, RGBA1};
+            %% Different sizes, should rescale on off them here
+            {#e3d_image{filename=File}=Map, _} ->
+                {File, e3d_image:expand_channel(b, e3d_image:convert(Map, g8, 1))}
+        end,
+    case E3d of
+        undefined -> undefined;
+        _ ->
+            Dir = filename:dirname(FN),
+            Ext = filename:extension(FN),
+            FileName = filename:basename(FN, Ext),
+            SaveFile = filename:join(Dir, FileName ++ "_met_rough" ++ Ext),
+            RGB = e3d_image:convert(E3d, r8g8b8),
+            wings_image:image_write([{image, RGB}, {filename, SaveFile}]),
+            RGB#e3d_image{filename=SaveFile}
+    end.
+
 exp_add_image(undefined, Data, _, _, GLTF) ->
     {Data, GLTF};
 exp_add_image(#e3d_image{filename=File}, Data, Key, gltf, GLTF0) ->
@@ -355,7 +456,7 @@ exp_add_image(#e3d_image{filename=File}, Data, Key, glb, GLTF0) ->
     {ok, ImageBin} = file:read_file(File),
     MType = case filename:extension(File) of
                 ".png" -> <<"image/png">>;
-                ".jpeg" -> <<"image/jpeg">>
+                ".jpg" -> <<"image/jpeg">>
             end,
     {BvId, GLTF1} = exp_add(ImageBin, bufferViews, GLTF0),
     {ImId, GLTF2} = exp_add(#{bufferView => BvId, mimeType=>MType}, images, GLTF1),
@@ -543,8 +644,8 @@ imp_mesh_data(What, AId, GLTF, Buffers) ->
     %% <<_:Offset0/binary,Buff:BlockSz/binary, _/binary>> = BuffBlock,
     <<_:Offset/binary, Buff:BlockSz/binary, _/binary>> = Buffer,
     ModType = case {What, Type} of
-                  {tx, <<"VEC2">>} ->
-                      <<"VEC2_FLIP_Y">>;
+                  {tx, <<"VEC2">>}        -> <<"VEC2_FLIP_Y">>;
+                  {tx, <<"VEC2_FLIP_Y">>} -> <<"VEC2">>;
                   _ -> Type
               end,
     Data = imp_buff(Buff, CType, ModType, Stride-CSz),
@@ -554,28 +655,119 @@ imp_mesh_data(What, AId, GLTF, Buffers) ->
 %%%%%%%%%
 
 imp_buff(Bin, ?GL_FLOAT, <<"SCALAR">>, Skip) ->
-    [A || <<A:32/float-little,_:Skip/binary>> <= Bin];
+    lists:reverse(imp_buff_1f(Bin, Skip, []));
 imp_buff(Bin, ?GL_FLOAT, <<"VEC2">>, Skip) ->
-    [{A,B} || <<A:32/float-little,B:32/float-little,_:Skip/binary>> <= Bin];
+    lists:reverse(imp_buff_2f(Bin, Skip, []));
 imp_buff(Bin, ?GL_FLOAT, <<"VEC2_FLIP_Y">>, Skip) ->
-    [{A,1.0-B} || <<A:32/float-little,B:32/float-little,_:Skip/binary>> <= Bin];
+    lists:reverse(imp_buff_2f_i(Bin, Skip, []));
 imp_buff(Bin, ?GL_FLOAT, <<"VEC3">>, Skip) ->
-    [{A,B,C} || <<A:32/float-little,B:32/float-little,
-                  C:32/float-little,_:Skip/binary>> <= Bin];
+    lists:reverse(imp_buff_3f(Bin, Skip, []));
 imp_buff(Bin, ?GL_FLOAT, <<"VEC4">>, Skip) ->
-    [{A,B,C,D} ||
-        <<A:32/float-little,B:32/float-little,
-          C:32/float-little,D:32/float-little,_:Skip/binary>> <= Bin];
+    lists:reverse(imp_buff_4f(Bin, Skip, []));
 imp_buff(Bin, ?GL_BYTE, <<"SCALAR">>, Skip) ->
-    [A || <<A:8/little, _:Skip/binary>> <= Bin];
+    lists:reverse(imp_buff_1b(Bin, Skip, []));
 imp_buff(Bin, ?GL_UNSIGNED_BYTE, <<"SCALAR">>, Skip) ->
-    [A || <<A:8/little-unsigned, _:Skip/binary>> <= Bin];
+    lists:reverse(imp_buff_1ub(Bin, Skip, []));
 imp_buff(Bin, ?GL_SHORT, <<"SCALAR">>, Skip) ->
-    [A || <<A:16/little, _:Skip/binary>> <= Bin];
+    lists:reverse(imp_buff_1s(Bin, Skip, []));
 imp_buff(Bin, ?GL_UNSIGNED_SHORT, <<"SCALAR">>, Skip) ->
-    [A || <<A:16/little-unsigned, _:Skip/binary>> <= Bin];
+    lists:reverse(imp_buff_1us(Bin, Skip, []));
 imp_buff(Bin, ?GL_UNSIGNED_INT, <<"SCALAR">>, Skip) ->
-    [A || <<A:32/little-unsigned, _:Skip/binary>> <= Bin].
+    lists:reverse(imp_buff_1ui(Bin, Skip, [])).
+
+imp_buff_1f(Bin, Skip, Acc) ->
+    case Bin of
+        <<A:32/float-little,_:Skip/binary, Rest/binary>> ->
+            imp_buff_1f(Rest, Skip, [A|Acc]);
+        <<A:32/float-little, _/binary>> when Skip > 0 ->
+            [A|Acc];
+        <<>> -> Acc
+    end.
+
+imp_buff_2f(Bin, Skip, Acc) ->
+    case Bin of
+        <<A:32/float-little,B:32/float-little,_:Skip/binary, Rest/binary>> ->
+            imp_buff_2f(Rest, Skip, [{A,B}|Acc]);
+        <<A:32/float-little,B:32/float-little, _/binary>> when Skip > 0 ->
+            [{A,B}|Acc];
+        <<>> -> Acc
+    end.
+
+imp_buff_2f_i(Bin, Skip, Acc) ->
+    case Bin of
+        <<A:32/float-little,B:32/float-little,_:Skip/binary, Rest/binary>> ->
+            imp_buff_2f_i(Rest, Skip, [{A,1.0-B}|Acc]);
+        <<A:32/float-little,B:32/float-little, _/binary>> when Skip > 0 ->
+            [{A,1.0-B}|Acc];
+        <<>> -> Acc
+    end.
+
+imp_buff_3f(Bin, Skip, Acc) ->
+    case Bin of
+        <<A:32/float-little,B:32/float-little,C:32/float-little,
+          _:Skip/binary, Rest/binary>> ->
+            imp_buff_3f(Rest, Skip, [{A,B,C}|Acc]);
+        <<A:32/float-little,B:32/float-little,C:32/float-little,
+          _/binary>> when Skip > 0 ->
+            [{A,B,C}|Acc];
+        <<>> -> Acc
+    end.
+
+imp_buff_4f(Bin, Skip, Acc) ->
+    case Bin of
+        <<A:32/float-little,B:32/float-little,C:32/float-little,D:32/float-little,
+          _:Skip/binary, Rest/binary>> ->
+            imp_buff_4f(Rest, Skip, [{A,B,C,D}|Acc]);
+        <<A:32/float-little,B:32/float-little,C:32/float-little,D:32/float-little,
+          _/binary>> when Skip > 0 ->
+            [{A,B,C,D}|Acc];
+        <<>> -> Acc
+    end.
+
+imp_buff_1b(Bin, Skip, Acc) ->
+    case Bin of
+        <<A:8,_:Skip/binary, Rest/binary>> ->
+            imp_buff_1b(Rest, Skip, [A|Acc]);
+        <<A:8, _/binary>> when Skip > 0 ->
+            [A|Acc];
+        <<>> -> Acc
+    end.
+
+imp_buff_1ub(Bin, Skip, Acc) ->
+    case Bin of
+        <<A:8/little-unsigned,_:Skip/binary, Rest/binary>> ->
+            imp_buff_1ub(Rest, Skip, [A|Acc]);
+        <<A:8/little-unsigned, _/binary>> when Skip > 0 ->
+            [A|Acc];
+        <<>> -> Acc
+    end.
+
+imp_buff_1s(Bin, Skip, Acc) ->
+    case Bin of
+        <<A:16/little,_:Skip/binary, Rest/binary>> ->
+            imp_buff_1s(Rest, Skip, [A|Acc]);
+        <<A:16/little, _/binary>> when Skip > 0 ->
+            [A|Acc];
+        <<>> -> Acc
+    end.
+
+imp_buff_1us(Bin, Skip, Acc) ->
+    case Bin of
+        <<A:16/little-unsigned,_:Skip/binary, Rest/binary>> ->
+            imp_buff_1us(Rest, Skip, [A|Acc]);
+        <<A:16/little-unsigned, _/binary>> when Skip > 0 ->
+            [A|Acc];
+        <<>> -> Acc
+    end.
+
+imp_buff_1ui(Bin, Skip, Acc) ->
+    case Bin of
+        <<A:32/little-unsigned,_:Skip/binary, Rest/binary>> ->
+            imp_buff_1ui(Rest, Skip, [A|Acc]);
+        <<A:32/little-unsigned, _/binary>> when Skip > 0 ->
+            [A|Acc];
+        <<>> -> Acc
+    end.
 
 size(?GL_FLOAT, <<"SCALAR">>) -> 4;
 size(?GL_FLOAT, <<"VEC2">>) -> 8;
@@ -673,7 +865,7 @@ imp_save_image({Dir, File}, Bin) ->
     case file:write_file(filename:join(Dir, File), Bin) of
         ok -> File;
         {error, _} ->
-            TmpDir = filename:basedir(user_cache, "wings3D"),
+            TmpDir = wings_u:basedir(user_cache),
             TempFile = filename:join(TmpDir, File),
             _ = file:write_file(TempFile, Bin),
             TempFile
@@ -685,46 +877,47 @@ imp_save_filename(File0, ImageType, Ext) ->
     File = lists:flatten(io_lib:format("~ts_~ts.~s",[OrigName, ImageType, Ext])),
     {Dir, File}.
 
-
 make_mats(#{materials:=Ms0}=GLTF, File) ->
+    Dir = filename:dirname(File),
     DefName = filename:rootname(filename:basename(File)),
     Ms = mat_add_names(Ms0, 0, DefName),
-    GLMs = [make_mat(Mat, GLTF) || Mat <- Ms],
+    GLMs = [make_mat(Mat, GLTF, Dir) || Mat <- Ms],
     {GLMs, GLTF#{materials:=Ms}};
 make_mats(GLTF, _) ->
     {[], GLTF}.
 
-make_mat(#{name:=Name}=Mat, GLTF) ->
+make_mat(#{name:=Name}=Mat, GLTF, Dir) ->
     case maps:get(pbrMetallicRoughness, Mat, none) of
         none ->
             Diffuse = [1.0,1.0,1.0,1.0],
             DiffuseTx = {diffuse, none},
             MetalTx = {metallic, none},
+            RoughTx = {roughness, none},
             MetalF = 1.0,
             RoughF = 0.9;
         Pbr ->
-            Diffuse = maps:get(baseColorFactor, Pbr, [1.0,1.0,1.0,1.0]),
+            Diffuse = [float(V) || V <- maps:get(baseColorFactor, Pbr, [1.0,1.0,1.0,1.0])],
             DiffuseTx = {diffuse, get_texture(baseColorTexture, Pbr, GLTF)},
-            MetalTx = {metallic, get_texture(metallicRoughnessTexture, Pbr, GLTF)},
-            MetalF = maps:get(metallicFactor, Pbr, 1.0),
-            RoughF = maps:get(roughnessFactor, Pbr, 0.9)
+            {Mtx,Rtx} = split_tx(get_texture(metallicRoughnessTexture, Pbr, GLTF), Dir),
+            MetalTx = {metallic, Mtx},
+            RoughTx = {roughness, Rtx},
+            MetalF = float(maps:get(metallicFactor, Pbr, 1.0)),
+            RoughF = float(maps:get(roughnessFactor, Pbr, 0.9))
     end,
 
     NormalTx = {normal, get_texture(normalTexture, Mat, GLTF)},
-    Emission = maps:get(emmissiveFactor, Mat, [0.0,0.0,0.0]),
+    Emission = [float(V) || V <- maps:get(emmissiveFactor, Mat, [0.0,0.0,0.0])],
     EmissionTx = {emission, get_texture(emissiveTexture, Mat, GLTF)},
     OccTx = {occlusion, get_texture(occlusionTexture, Mat, GLTF)},
-    Txs = [NormalTx, DiffuseTx, MetalTx, EmissionTx, OccTx],
+    Txs = [NormalTx, DiffuseTx, MetalTx, RoughTx, EmissionTx, OccTx],
     Maps = case [Map || {_, F} = Map <- Txs, F =/= none] of
                [] -> [];
                Maps0 -> [{maps, Maps0}]
            end,
     DiffuseC = list_to_tuple(Diffuse),
-    Specular = wings_color:mix(MetalF, DiffuseC, {0.8,0.8,0.8,0.8}),
     DefList = [{diffuse, DiffuseC},
-               {ambient, {0.0,0.0,0.0,0.0}},
-               {specular, Specular},
-               {shininess, 1.0-RoughF},
+               {roughness, RoughF},
+               {metallic, MetalF},
                {emission, list_to_tuple(Emission)}],
     {binary_to_atom(Name, utf8), [{opengl, DefList}|Maps]}.
 
@@ -743,6 +936,29 @@ get_texture(Name, Mat, #{read_image:=Get} = GLTF) ->
             end
     end.
 
+%% Need to split combined image file to two files
+split_tx(none, _) -> {none, none};
+split_tx(OrigFileName, Dir0) ->
+    Dir = case filename:dirname(OrigFileName) of
+              "." -> Dir0;
+              Dir1 -> Dir1
+          end,
+    Ext = filename:extension(OrigFileName),
+    File = filename:basename(OrigFileName, Ext),
+    E3d = wings_image:image_read([{filename, OrigFileName}, {opt_dir, Dir0}]),
+    try
+        Metallic = e3d_image:channel(b,E3d),
+        MetFile = filename:join(Dir, File++"_met" ++ Ext),
+        wings_image:image_write([{filename, MetFile}, {image, Metallic}]),
+        Roughness = e3d_image:channel(g,E3d),
+        RoughFile = filename:join(Dir, File++"_rough" ++ Ext),
+        wings_image:image_write([{filename, RoughFile}, {image, Roughness}]),
+        {MetFile, RoughFile}
+    catch _:Reason ->
+            ?dbg("Internal error: ~p ~p~n", [Reason, erlang:get_stacktrace()]),
+            {none, none}
+    end.
+
 mat_add_names([#{name:=_}=M|Ms], N, DefName) ->
     [M|mat_add_names(Ms, N, DefName)];
 mat_add_names([M|Ms], N, DefName) ->
