@@ -82,6 +82,7 @@ init_part2(Frame, GeomGL) ->
     %% and before the others that use gl functions
     wings_text:init(),
     wings_wm:init(Frame),
+    wings_develop:init(),
     check_requirements(),
 
     wings_color:init(),
@@ -89,13 +90,14 @@ init_part2(Frame, GeomGL) ->
 
     wings_camera:init(),
     wings_vec:init(),
+    wings_shaders:init(),
+    wings_light:init(),
 
     wings_view:init(),
     wings_u:caption(St),
     wings_file:init_autosave(),
     wings_dialog:init(),
     wings_job:init(),
-    wings_develop:init(),
     wings_tweak:init(),
 
     open_file(),
@@ -254,6 +256,7 @@ init_opengl(St) ->
     wings_render:init(),
     wings_dl:init(),
     wings_draw:refresh_dlists(St),
+    wings_light:init_opengl(),
     keep.
 
 redraw(St) ->
@@ -471,12 +474,8 @@ handle_event_3({external,not_possible_to_save_prefs}, _St) ->
     wings_help:not_possible_to_save_prefs();
 handle_event_3({external, win32_start_maximized}, _St) ->
     keep;
-handle_event_3({external, Fun}, St)
-  when is_function(Fun) ->
+handle_event_3({external, Fun}, St) when is_function(Fun) ->
     Fun(St);
-handle_event_3({external,Op}, St) ->
-    wpa:handle_external(Op,St),
-    keep;
 handle_event_3(#wx{event=#wxDropFiles{files=Fs0}}, St0) ->
     Send = fun(Op) -> wings ! {action, {file, Op}} end,
     ImgFms = wings_image:image_formats(),
@@ -490,16 +489,16 @@ handle_event_3(#wx{event=#wxDropFiles{files=Fs0}}, St0) ->
               (File, false) -> Send({merge, File}), false
            end,
     lists:foldl(Open, DoOpen, Objects),
-    case Fs2 of
-        [] -> keep;
-        [File] -> wings_u:error_msg(?__(1,"Unknown file format: ~s"), [filename:basename(File)]);
-        [_|_]  -> wings_u:error_msg(?__(2,"Unknown file formats"))
-    end;
+    Send({import_files, Fs2}),
+    keep;
 handle_event_3(ignore, _St) ->
     keep;
 handle_event_3({system,_,_}, _St) ->
     %% observer or other system message
     keep;
+handle_event_3({command_error, Error}, _St) ->
+    wings_u:error_msg(Error),
+    keep;
 handle_event_3({adv_menu_abort, Ev}, _St) ->
     This = wings_wm:actual_focus_window(),
     wings_wm:send(This,Ev);
@@ -529,7 +528,7 @@ handle_popup_event(Ev, Xglobal, Yglobal, St0) ->
 
 filter_format(File, Formats) ->
     Ext = filename:extension(File),
-    lists:keymember(Ext, 1, Formats).
+    lists:keymember(string:lowercase(Ext), 1, Formats).
 
 info_line() ->
     case wings_pref:get_value(tweak_active) of
@@ -717,7 +716,7 @@ repeatable(Mode, Cmd) ->
     
     %% No more commands are safe in body mode.
     {_,_} when Mode == body -> no;
-    {_,{flatten,_}=C} when Mode == vertex; Mode == face -> {Mode,C};
+    {_,{flatten,_}=C} when Mode == vertex; Mode == face; Mode == edge -> {Mode,C};
     {_,dissolve} when Mode == vertex -> no;
     {_,dissolve=C} -> {Mode,C};
     {_,bevel=C} -> {Mode,C};
@@ -803,6 +802,7 @@ command_1({edit,{preferences,Pref}}, St) ->
 command_1({edit,{theme,Theme}}, St) ->
     wings_pref:pref({load,Theme,St}),
     wings_frame:update_theme(),
+    wings_status:update_theme(),
     keep;
 
 %% Select menu.
@@ -1243,7 +1243,6 @@ get_mode_restriction() ->
 
 %%%% Get Objects Info and display in a table, one row per object.
 object_info(St) ->
-    #st{shapes=Shapes} = St,
     HRow = {
       ?__(1,"#"),
       ?__(2,"Name"),
@@ -1253,10 +1252,15 @@ object_info(St) ->
       ?__(6,"Edges"),
       ?__(7,"Faces"),
       ?__(8,"Verts") },
-    case gb_trees:is_empty(Shapes) of
-        true -> wings_u:error_msg(?__(9,"No objects in scene"));
-        false ->
-            Rows = [get_object_info(Id, Shapes) || Id <- gb_trees:keys(Shapes)],
+    MF = fun(#{id:=Id,name:=Name},#we{}=We) ->
+                 get_object_info(Id, Name, We)
+         end,
+    RF = fun(Obj, Acc) -> [Obj|Acc] end,
+    case wings_obj:dfold(MF, RF, [], St) of
+        [] ->
+            wings_u:error_msg(?__(9,"No objects in scene"));
+        [_|_]=Rows0 ->
+            Rows = lists:sort(Rows0),
 	    Header = list_to_tuple([{H,H}|| H <- tuple_to_list(HRow)]),
             ColumnWidthList = column_widths([Header|Rows]),
             Qs = [{table,[HRow|Rows],
@@ -1266,42 +1270,38 @@ object_info(St) ->
             wings_dialog:dialog(?__(10,"Scene Info: "), Qs, Ask)
     end.
 
+get_object_info(Id, Name, #we{es=Etab,fs=Ftab,vp=VPos}=We) ->
+    Area      =  wings_we:surface_area(We),
+    Volume    =  wings_we:volume(We),
+    Perimeter =  wings_we:perimeter(We),
+    NEdge   = wings_util:array_entries(Etab),
+    NFace   = gb_trees:size(Ftab),
+    NVertex =  wings_util:array_entries(VPos),
+    Info = [Id,Name,Area,Perimeter,Volume,NEdge,NFace,NVertex],
+    list_to_tuple([{X,to_string(X)} || X <- Info]).
 
-get_object_info(Id, Shapes) ->
-    #we{id=Id,name=Name,es=Etab0,fs=Ftab0,vp=VPos0} = We0 = gb_trees:get(Id, Shapes),
-    Area      =  wings_we:surface_area(We0),
-    Volume    =  wings_we:volume(We0),
-    Perimeter =  wings_we:perimeter(We0),
-    ToString = fun(Item) ->
-		       case Item of
-			   Item when is_float(Item) ->
-			       lists:flatten(io_lib:format("~.4f", [Item]));
-			   Item when is_integer(Item) ->
-			       integer_to_list(Item);
-			   Item when is_list(Item) ->
-			       Item
-		       end
-	       end,
-    NEdge   = wings_util:array_entries(Etab0),
-    NFace   = gb_trees:size(Ftab0),
-    NVertex =  wings_util:array_entries(VPos0),
-    list_to_tuple([{X,ToString(X)}||X<-[Id,Name,Area,Perimeter,Volume,NEdge,NFace,NVertex]]).
+to_string(Item) ->
+    case Item of
+        Item when is_float(Item) ->
+            lists:flatten(io_lib:format("~.4f", [Item]));
+        Item when is_integer(Item) ->
+            integer_to_list(Item);
+        Item when is_list(Item) ->
+            Item
+    end.
 
-
-
-%% get maximal column widths needed to display each column in characters.
+%% Get maximal column widths needed to display each column in characters.
 column_widths(Rows) ->
     [Row|_] = Rows,
-    NCols = size(Row),
-    MyAcc = fun(Col,Acc) ->
-		    Temp = [ begin
-				 {_,A} = element(Col,RowTuple),
-				 length(A)
-			     end || RowTuple <- Rows],
-		    Mx = lists:max(Temp), % add a bit of padding
-		    [Mx|Acc]
-	    end,
-    lists:foldr(MyAcc,[], lists:seq(1,NCols)).
+    NCols = tuple_size(Row),
+    F = fun(Col) ->
+                L = [begin
+                         {_,A} = element(Col, RowTuple),
+                         length(A)
+                     end || RowTuple <- Rows],
+                lists:max(L)
+        end,
+    [F(Col) || Col <- lists:seq(1, NCols)].
 %%%%
 
 highlight_aim_setup(St0) ->
