@@ -18,6 +18,7 @@
 -define(NEED_ESDL, 1).
 -include("wings.hrl").
 -import(lists, [foldl/3,mapfoldl/3,reverse/1,sort/1,seq/2]).
+-define(STR_LIGHT(A,B,Str), wings_lang:str({wings_light,A,B},Str)).
 
 menu(X, Y, St) ->
     Dir = wings_menu_util:directions(St),
@@ -61,6 +62,10 @@ menu(X, Y, St) ->
 	    {?__(18,"Separate"),separate,
 	     ?__(19,"Separate a combined objects into its components")},
 	    separator,
+	    {?__(200, "Boolean *TEST*"),
+             {bool, [{?__(201,"Union"),     add, ?__(202, "Add parts from intersecting objects")},
+                     {?__(203,"Intersect"), isect, ?__(204, "Make objects from the intersecting parts")},
+                     {?__(205,"Difference"), sub, ?__(206, "Subtract objects from secondary selection")}]}},
 	    {?__(20,"Weld"),weld,
 	     ?__(21,"Merge pair of faces that are nearly coincident"),
 	     [option]},
@@ -139,7 +144,12 @@ mode_dependent(St) ->
             (_, _) -> mixed
          end,
     Kind = wings_sel:dfold(MF, RF, [], St),
-    Tail = vertex_color_item(Kind),
+    Tail0 = vertex_color_item(Kind),
+    Tail =
+	case wings_sel:selected_ids(St) of
+	    [Id] -> arealight_edit(Id, Tail0);
+	    _ -> Tail0
+	end,
     arealight_conv(Kind, Tail).
 
 vertex_color_item(object) ->
@@ -147,6 +157,10 @@ vertex_color_item(object) ->
       ?__(3,"Apply vertex colors to selected objects")}];
 vertex_color_item(_) -> [].
 
+arealight_edit(Id, T) ->
+    [{?STR_LIGHT(menu,16,"Edit Properties..."),{edit_arealight,Id},
+      ?STR_LIGHT(menu,17,"Edit light properties")}|T].
+
 arealight_conv(arealight, T) ->
     [{?__(1,"Area Light to Object"),from_arealight,
       ?__(2,"Convert selected area lights to objects")}|T];
@@ -213,6 +227,8 @@ command(to_arealight, St) ->
     to_arealight(St);
 command(from_arealight, St) ->
     from_arealight(St);
+command({edit_arealight,Id}, St) ->
+    wings_light:command({edit,Id},St);
 command({vertex_attributes,materials_to_colors}, St) ->
     {save_state,materials_to_colors(St)};
 command({vertex_attributes,colors_to_materials}, St) ->
@@ -223,6 +239,14 @@ command({vertex_attributes,remove_uv_coordinates}, St) ->
     {save_state,va_remove(uv, St)};
 command({vertex_attributes,remove_all_attributes}, St) ->
     {save_state,va_remove(all, St)};
+command({bool,add}, St0) ->
+    St1 = ?SLOW(wings_sel:valid_sel(wings_bool:add(St0))),
+    {save_state, wings_obj:recreate_folder_system(St1)};
+command({bool,isect}, St0) ->
+    St1 = ?SLOW(wings_sel:valid_sel(wings_bool:isect(St0))),
+    {save_state, wings_obj:recreate_folder_system(St1)};
+command({bool,sub}, St0) ->
+    wings_bool:sub(St0);
 command({weld,Ask}, St) ->
     ?SLOW(weld(Ask, St));
 command(vertex_color, St) ->
