@@ -2,8 +2,8 @@
 %%  wings_cc_ref.erl --
 %%
 %%     This module contains helper functions for OpenCL.
-%% 
-%%  Copyright (c) 2010-2011 Dan Gudmundsson
+%%
+%%  Copyright (c) 2010-2018 Dan Gudmundsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -14,38 +14,66 @@
 -module(wings_cl).
 -include_lib("cl/include/cl.hrl").
 -compile([{nowarn_deprecated_function, {erlang,get_stacktrace,0}}]).
+-include_lib("wings/e3d/e3d_image.hrl").
 
--export([is_available/0,
-	 setup/0, stop/1, compile/2, compile/3,
+-export([is_available/1, setup/0, stop/1, working/0,
+         compile/2, compile/3,
 	 %% Queries
 	 get_context/1, get_device/1, get_queue/1, get_vendor/1,
-	 have_image_support/1,
+	 have_image_support/1, is_kernel/2,
 
-	 buff/2, buff/3, image/4, image/5, write/3, read/4,
+	 buff/2, buff/3, write/3, read/4, fill/4,
+         image/2, image/3, read_img/6,
 	 cast/4, cast/5, tcast/4, tcast/5, set_args/3,
 	 get_wg_sz/2, set_wg_sz/3,
 	 get_lmem_sz/2
 	]).
 
--record(cli, {context, kernels=[], q, cl, device}).
+-record(cli, {context, kernels=#{}, q, cl, device}).
 -record(kernel, {name, id, wg}).
 
-is_available() ->
-    try 
+-on_load(init_develop/0).
+
+%%
+%% Add path to be used during development if cl is downloaded as a dependency
+%%
+init_develop() ->
+    Top = filename:dirname(filename:dirname(code:which(?MODULE))),
+    _ = code:add_patha(filename:join(Top, "_deps/cl/ebin")),
+    ok.
+
+%%
+%% A call to wings_cl:is_available(true) must be followed by a call to wings_cl:working()
+%%
+is_available(Write) ->
+    try
 	true == erlang:system_info(smp_support) orelse throw({error, no_smp_support}),
+        Type = wings_pref:get_value(cl_type, gpu),
+        Type =:= gpu orelse Type =:= cpu orelse throw({error, opencl_user_disabled}),
+        case file:read_file_info(temp_file()) of
+            {ok, _} -> throw({error, {opencl_failed_previously, temp_file()}});
+            {error,_} -> ok
+        end,
+        Write andalso file:write_file(temp_file(), <<"Delete me if OpenCL is working">>),
 	ok == cl:start() orelse throw({error, no_opencl_loaded}),
 	{ok, Ps} = cl:get_platform_ids(),
 	[] /= Ps
-    catch _:Reason ->
-	    io:format("OpenCL not available ~p ~n",[Reason]),
+    catch throw:{error, {opencl_failed_previously, _} = Reason} ->
+	    io:format("OpenCL not available: ~p ~n",[Reason]),
+	    false;
+          throw:{error, Reason} ->
+            io:format("OpenCL not available: ~p ~n",[Reason]),
+            working(), %% Does not crash
+	    false;
+          _:Reason ->
+            io:format("OpenCL not available: ~p ~n",[Reason]),
 	    false
     end.
 
-
 %% setup() -> cli().
 setup() ->
     Prefered = wings_pref:get_value(cl_type, gpu),
-    Other = [gpu,cpu] -- [Prefered],
+    [Other] = [gpu,cpu] -- [Prefered],
     {Use,CL} = case clu:setup(Prefered) of
                    {error, _} ->
                        case clu:setup(Other) of
@@ -64,6 +92,14 @@ setup() ->
 stop(#cli{cl=CL}) ->
     clu:teardown(CL).
 
+%% Call me if OpenCL initiation worked as expected
+%% with or without OpenCL.
+working() ->
+    _ = file:delete(temp_file()),
+    ok.
+
+temp_file() ->
+    filename:join(wings_u:basedir(user_cache), "opencl_tmp.txt").
 
 %% compile(File,cli()) -> cli().
 %%
@@ -77,9 +113,14 @@ compile(File = [A|_], Defs, CLI) when is_integer(A) ->
 compile(Files, Defs, CLI) ->
     compile_1(Files, Defs, CLI).
 
+is_kernel(Kernel, #cli{kernels=Ks}) ->
+    maps:is_key(Kernel, Ks);
+is_kernel(_, _) ->
+    false.
+
 compile_1(Files, Defs, CLI = #cli{cl=CL, device=Device, kernels=Kernels0}) ->
     Dir = filename:join(code:lib_dir(wings),"shaders"),
-    Bins = lists:map(fun(File) ->
+    SrcBins = lists:map(fun(File) ->
 			     AbsFile = filename:join([Dir, File]),
 			     case file:read_file(AbsFile) of
 				 {ok, Bin} -> {AbsFile, Bin};
@@ -87,16 +128,18 @@ compile_1(Files, Defs, CLI = #cli{cl=CL, device=Device, kernels=Kernels0}) ->
 				     error({error,{Reason,AbsFile}})
 			     end
 		     end, Files),
-    {ok, Program} = build_source(CL, Bins, Defs),
+    {ok, Program} = build_source(CL, SrcBins, Defs),
     {ok, MaxWGS} = cl:get_device_info(Device, max_work_group_size),
     {ok, KernelsIds} = cl:create_kernels_in_program(Program),
     Kernels = [kernel_info(K,Device, MaxWGS) || K <- KernelsIds],
     cl:release_program(Program),
-    CLI#cli{kernels=Kernels++Kernels0}.
+    CLI#cli{kernels=maps:merge(Kernels0, maps:from_list(Kernels))}.
 
 build_source(E, Sources, Defines) ->
-    Source = [Bin || {_, Bin} <- Sources],
-    {ok,Program} = cl:create_program_with_source(E#cl.context,Source),
+    {Files, SourceBin} = lists:unzip(Sources),
+    {ok,Program} = cl:create_program_with_source(E#cl.context,SourceBin),
+    %% Debug (on Intel cpu) with
+    %% case cl:build_program(Program, E#cl.devices, "-g -s " ++ hd(Files) ++ " " ++ Defines) of
     case cl:build_program(Program, E#cl.devices, Defines) of
 	ok ->
 	    Status = [{Dev, cl:get_program_build_info(Program, Dev, status)}
@@ -108,18 +151,15 @@ build_source(E, Sources, Defines) ->
 		    {ok,Program};
 		Errs ->
 		    ErrDevs = [Dev || {Dev, _} <- Errs],
-		    display_error(?LINE, Program, Sources, Defines, ErrDevs)
+		    display_error(?LINE, Program, Files, Defines, ErrDevs)
 	    end;
 	_Error ->
-	    display_error(?LINE, Program, Sources, Defines, E#cl.devices)
+	    display_error(?LINE, Program, Files, Defines, E#cl.devices)
     end.
 
-
-
-display_error(Line, Program, Sources, _Defines, DeviceList) ->
-    SFs = [S || {S,_} <- Sources],
+display_error(Line, Program, Files, _Defines, DeviceList) ->
     io:format("~n~p:~p: Error in source file(s):~n",[?MODULE, Line]),
-    [io:format(" ~s~n",[Source]) || Source <- SFs],
+    [io:format(" ~s~n",[Source]) || Source <- Files],
     lists:map(fun(Device) ->
 		      {ok, DevName} = cl:get_device_info(Device, name),
 		      io:format("Device: ~s~n",[DevName]),
@@ -144,19 +184,25 @@ display_error(Line, Program, Sources, _Defines, DeviceList) ->
     %% io:format("Debug written to: ~s ~n", [filename:join(DbgOutDir, "cl_compilation_fail.cl")]),
     exit({error, build_program_failure}).
 
-
 kernel_info(K,Device,MaxWGS) ->
-    {ok, WG} = cl:get_kernel_workgroup_info(K, Device, work_group_size),
-    {ok, CWG} = cl:get_kernel_workgroup_info(K, Device, compile_work_group_size),
-    {ok, Name} = cl:get_kernel_info(K, function_name),
+    WG = case cl:get_kernel_workgroup_info(K, Device, work_group_size) of
+             {error, _} -> MaxWGS div 2;
+             {ok, Val0} -> Val0
+         end,
+    CWG = case cl:get_kernel_workgroup_info(K, Device, compile_work_group_size) of
+              {error, _} -> [0,0,0];
+              {ok, Val1} -> Val1
+          end,
+    {ok, Name0} = cl:get_kernel_info(K, function_name),
+    Name = list_to_atom(Name0),
     %% io:format("~s WG sizes ~p ~p~n", [Name, WG, WG1]),
     case CWG of
 	[0,0,0] ->
-	    #kernel{name=list_to_atom(Name), wg=min(WG,MaxWGS), id=K};
+	    {Name, #kernel{name=Name, wg=min(WG,MaxWGS), id=K}};
 	[Max,1,1] ->
-	    #kernel{name=list_to_atom(Name), wg=min(Max,MaxWGS), id=K};
+	    {Name, #kernel{name=Name, wg=min(Max,MaxWGS), id=K}};
 	MaxD ->
-	    #kernel{name=list_to_atom(Name), wg=MaxD, id=K}
+	    {Name, #kernel{name=Name, wg=MaxD, id=K}}
     end.
 
 get_context(#cli{context=Context}) ->
@@ -176,41 +222,41 @@ get_vendor(#cli{device=ClDev}) ->
     Vendor.
 
 set_args(Name, Args, #cli{kernels=Ks}) ->
-    #kernel{id=K} = lists:keyfind(Name, 2, Ks),
+    #kernel{id=K} = maps:get(Name, Ks),
     set_args_1(Name, K, Args).
 
 get_lmem_sz(Name, #cli{kernels=Ks, device=Device}) ->
-    #kernel{id=Kernel} = lists:keyfind(Name, 2, Ks),
+    #kernel{id=Kernel} = maps:get(Name, Ks),
     {ok,Mem} = cl:get_kernel_workgroup_info(Kernel, Device, local_mem_size),
     Mem.
 
 get_wg_sz(Name, #cli{kernels=Ks}) ->
-    #kernel{wg=Wg} = lists:keyfind(Name, 2, Ks),
+    #kernel{wg=Wg} = maps:get(Name, Ks),
     Wg.
 
 set_wg_sz(Name, Wg, CL=#cli{kernels=Ks0}) ->
-    K  = lists:keyfind(Name, 2, Ks0),
-    Ks = lists:keyreplace(Name, 2, Ks0, K#kernel{wg=Wg}),
+    K  = maps:get(Name, Ks0),
+    Ks = Ks0#{Name:=K#kernel{wg=Wg}},
     CL#cli{kernels=Ks}.
 
 %% cast(Kernel, Args, NoInvocations, [Wait], cli()) -> Wait
 tcast(Name, No, Wait, #cli{q=Q, kernels=Ks}) ->
-    Kernel = lists:keyfind(Name, 2, Ks),
+    Kernel = maps:get(Name, Ks),
     Event = enqueue_kernel(No, twait(Wait), Q, Kernel),
     time_wait(Name, Q, Event),
     Event.
 cast(Name, No, Wait, #cli{q=Q, kernels=Ks}) ->
-    Kernel = lists:keyfind(Name, 2, Ks),
+    Kernel = maps:get(Name, Ks),
     enqueue_kernel(No, Wait, Q, Kernel).
 
 tcast(Name, Args, No, Wait, #cli{q=Q, kernels=Ks}) ->
-    Kernel = #kernel{id=K} = lists:keyfind(Name, 2, Ks),
+    Kernel = #kernel{id=K} = maps:get(Name, Ks),
     set_args_1(Name, K, Args),
     Event = enqueue_kernel(No, twait(Wait), Q, Kernel),
     time_wait(Name, Q, Event),
     Event.
 cast(Name, Args, No, Wait, #cli{q=Q, kernels=Ks}) ->
-    Kernel = #kernel{id=K} = lists:keyfind(Name, 2, Ks),
+    Kernel = #kernel{id=K} = maps:get(Name, Ks),
     set_args_1(Name, K, Args),
     enqueue_kernel(No, Wait, Q, Kernel).
 
@@ -233,23 +279,55 @@ buff(Bin, Type, #cli{context=Context})
     {ok, Buff} = cl:create_buffer(Context, Type, byte_size(Bin), Bin),
     Buff.
 
-image(Bin, Dim, Format, CL) ->
-    image(Bin, Dim, Format, [read_only, copy_host_ptr], CL).
-image(Bin, {W,H}, #cl_image_format{} = Format, Alloc, #cli{context=Context})
-  when is_binary(Bin) ->
-    {ok, Buff} = cl:create_image2d(Context, Alloc, Format, W, H, 0, Bin),
+image(#e3d_image{image=Img}=E3d, CL)
+  when Img =:= <<>>; Img =:= undefined ->
+    image(E3d, [], CL);
+image(#e3d_image{}=E3d, CL) ->
+    image(E3d, [read_only, copy_host_ptr], CL).
+image(#e3d_image{width=W,height=H, type=Type, bytes_pp=Bpp, image=Bin},
+      Alloc, #cli{context=Context}) ->
+    Format = image2d_format(Type, Bpp),
+    Desc = image2d_desc(W,H),
+    {ok, Buff} = cl:create_image(Context, Alloc, Format, Desc, Bin),
     Buff.
 
+image2d_desc(W,H) ->
+    #cl_image_desc{
+       image_type = image2d,
+       image_width = W,     image_height = H,
+       image_depth = 1,     image_array_size = 1,
+       image_row_pitch = 0, image_slice_pitch = 0,
+       buffer = get('this_fools_dialyzer_bad_spec_in_cl.hrl')
+      }.
+
+image2d_format(_, 1) ->
+    #cl_image_format{cl_channel_order = r, cl_channel_type= unorm_int8};
+%% image2d_format(r8g8b8, 3) ->  %% Not allowed in OpenCL 1.2
+%%     #cl_image_format{cl_channel_order = rgb, cl_channel_type= unorm_int8};
+image2d_format(r8g8b8a8, 4) ->
+    #cl_image_format{cl_channel_order = rgba, cl_channel_type= unorm_int8}.
+
 %% write(CLMem, Bin, cli()) -> Wait
 write(CLMem, Bin, #cli{q=Q}) ->
     {ok, W1} = cl:enqueue_write_buffer(Q, CLMem, 0, byte_size(Bin), Bin, []),
     W1.
 
+%fill(CLMem, Pattern, Sz, #cli{q=Q}) when is_binary(Pattern) ->
+fill(CLMem, <<0:(32*4)>>, Sz, #cli{q=Q}) ->
+    Bin = <<0:(8*Sz)>>,
+    {ok, W1} = cl:enqueue_write_buffer(Q, CLMem, 0, Sz, Bin, []),
+    %% {ok, W1} = cl:enqueue_fill_buffer(Q, CLMem, Pattern, 0, Sz, []),
+    W1.
+
 %% read(CLMem, Sz, [Wait], cli()) -> Wait
 read(CLMem, Sz, Wait, #cli{q=Q}) ->
     {ok, W} = cl:enqueue_read_buffer(Q,CLMem,0,Sz, Wait),
     W.
 
+read_img(CLImg, W, H, 4=Bpp, Wait, #cli{q=Q}) ->
+    {ok, Res} = cl:enqueue_read_image(Q, CLImg, [0,0], [W,H], W*Bpp, 0, Wait),
+    Res.
+
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 set_args_1(Name, K, Args) ->
@@ -266,7 +344,7 @@ enqueue_kernel(No, Wait, Q, #kernel{id=K, wg=WG0}) ->
     case Wait of
 	nowait ->
 	    ok = cl:nowait_enqueue_nd_range_kernel(Q,K,GWG,WG,[]);
-	    _ ->
+        _ ->
 	    {ok, Event} = cl:enqueue_nd_range_kernel(Q,K,GWG,WG,Wait),
 	    Event
     end.
@@ -280,10 +358,10 @@ calc_wg(No, WG)
 calc_wg(No, WG)
   when is_integer(No), is_integer(WG) ->
     {[(1+(No div WG))*WG], [WG]};
-calc_wg([WH|WT], [H|T]) ->
-    {[CW], [CS]} = calc_wg(WH,H),
-    {CT, CST} = calc_wg(WT,T),
-    {[CW|CT], [CS|CST]};
+calc_wg([W1, W2], WG) ->
+    {[C1], [CS]} = calc_wg(W1,WG),
+    {[C2], _} = calc_wg(W2,WG),
+    {[C1, C2], [CS, 1]};  %% Needed for old gfx cards
 calc_wg([], [1]) ->
     {[],[]};
 calc_wg([], [H]) ->
