@@ -15,7 +15,7 @@
 -export([import/2,merge/2,export/2]).
 
 -include("wings.hrl").
--include("e3d_image.hrl").
+-include_lib("wings/e3d/e3d_image.hrl").
 -import(lists, [sort/1,reverse/1,foldl/3,any/2,keymember/3,keyfind/3]).
 
 -define(WINGS_HEADER, "#!WINGS-1.0\r\n\032\04").
@@ -1294,7 +1294,7 @@ collect_sel(#st{selmode=Mode,sel=Sel0,ssels=Ssels}=St) ->
 
 collect_sel_groups([{{Mode,Name},Sel}|Gs], St, Acc0) ->
     Acc = [{Id,{Mode,gb_sets:to_list(Elems),{selection_group,Name}}} ||
-	      {Id,Elems} <- wings_sel:valid_sel(Sel, Mode, St)] ++ Acc0,
+	      {Id,Elems} <- wings_sel:valid_sel_groups(Sel, Mode, St)] ++ Acc0,
     collect_sel_groups(Gs, St, Acc);
 collect_sel_groups([], _, Acc) -> Acc.
 
