@@ -309,15 +309,9 @@ init(_Opts) ->
 
 	wxWindow:connect(Frame, close_window, [{callback, fun terminate_frame/2}]),
 	wxWindow:connect(Frame, command_menu_selected, []),
-	wxWindow:connect(Frame, activate, []),
-	case os:type() of
-	    {_, darwin} ->
-		Me = self(),
-		CB = fun(_,_) -> Me ! {init_menus, Frame} end,
-		wxWindow:connect(Frame, show, [{callback, CB}]);
-	    _ ->
-		init_menubar(Frame)
-	end,
+	wxWindow:connect(Frame, activate, [{skip, true}]),
+        init_menubar(Frame),
+        wxFrame:show(Frame),
 	Wins = #{frame=>Frame, ch=>Top#split{w1=Canvas}, szr=>Sizer,
 		 loose=>#{}, action=>undefined, op=>undefined},
 	Overlay = make_overlay(Frame),
@@ -341,7 +335,8 @@ make_splash(Canvas, Imgs) ->
 %%%%%%%%%%%%%%%%%%%%%%%
 
 handle_event(#wx{obj=Win, event=#wxMove{}}, State) ->
-    #wxMouseState{x=X,y=Y} = MS = wx_misc:getMouseState(),
+    MS = wx_misc:getMouseState(),
+    {X,Y} = wx_misc:getMousePosition(),
     %% io:format("Move ~p ~p ~p ~n", [Win, X,Y]),
     {noreply, preview_attach(stopped_moving(MS), {X,Y}, Win, State)};
 
@@ -406,7 +401,12 @@ handle_event(#wx{obj=Obj, event=#wxClose{}}, #state{windows=Wins}=State) ->
 	    catch _:_ ->
 		    wings_wm:psend(Name, close),
 		    {noreply, State}
-	    end
+	    end;
+	_ ->
+	    ?dbg("\nUnexpected window closing\n\tObj: ~p\n\tWins: ~p\n",[Obj,Wins]),
+	    wings_u:message("Unexpected window closing.\n" ++
+			    "Please check the log window and report the information there."),
+	    {noreply, State}
     end;
 handle_event(_Ev, State) ->
     %% io:format("~p:~p Got unexpected event ~p~n", [?MODULE,?LINE, _Ev]),
@@ -499,16 +499,6 @@ handle_cast({got_focus, Window, Props}, #state{toolbar=TB0}=State) ->
     TB = wings_toolbar:update({active, Window, ModeRest}, TB0),
     wings_status:active(Window),
     {noreply, update_active(Window, State#state{toolbar=TB})};
-handle_cast({init_menus, Frame}, State) ->
-    case os:type() of
-	{_, darwin} ->
-	    _MB = init_menubar(Frame),
-	    %io:format("HO ~p~n", [wxMenuBar:findItem(MB, ?wxID_OSX_HIDEOTHERS)]),
-	    %io:format("SA ~p~n", [wxMenuBar:findItem(MB, ?wxID_OSX_SHOWALL)]),
-	    ok;
-	_ -> ignore
-    end,
-    {noreply, State};
 
 handle_cast({set_title, Win, Title}, #state{windows=#{ch:=Root, loose:=Loose}=Wins} = State) ->
     case find_win(Win, Root) of
@@ -581,9 +571,13 @@ terminate_frame(_Ev, CB) ->
 update_active(Name, #state{active=Prev, windows=#{ch:=Root}}=State) ->
     ABG = wings_color:rgb4bv(wings_pref:get_value(title_active_color)),
     PBG = wings_color:rgb4bv(wings_pref:get_value(title_passive_color)),
+    TFG = wings_color:rgb4bv(wings_pref:get_value(title_text_color)),
     try
 	#win{bar={PBar,_}} = find_win(Prev, Root),
 	_ = wxWindow:getSize(PBar), %% Sync to check PBar validity
+	PChildren = wxWindow:getChildren(PBar),
+	[wxWindow:setForegroundColour(PChild, TFG) || PChild <- PChildren,
+	 wx:getObjectType(PChild) == wxWindow],
 	wxWindow:setBackgroundColour(PBar, PBG),
 	wxWindow:refresh(PBar)
     catch _:_ -> ignore
@@ -592,6 +586,9 @@ update_active(Name, #state{active=Prev, windows=#{ch:=Root}}=State) ->
 	false ->
 	    State#state{active=undefined};
 	#win{bar={ABar,_}} ->
+	    AChildren = wxWindow:getChildren(ABar),
+	    [wxWindow:setForegroundColour(AChild, TFG) || AChild <- AChildren,
+	     wx:getObjectType(AChild) == wxWindow],
 	    wxWindow:setBackgroundColour(ABar, ABG),
 	    wxWindow:refresh(ABar),
 	    State#state{active=Name}
@@ -691,7 +688,7 @@ attach_floating(true, Overlay, #{op:=#{mwin:=Frame, mpath:=Path}, loose:=Loose}=
 	    St = State#{loose:=maps:remove(Frame,Loose), action:=undefined, op:=undefined},
 	    DoWhileLocked = fun() -> attach_window(Path, Frame, Window, St) end,
 	    After = fun() ->
-			    timer:sleep(200),
+			    timer:sleep(200), %% give wx time to update windows on X11
 			    wings_io:reset_video_mode_for_gl(0,0),
                             inform_parent_changed(Win)
 		    end,
@@ -894,12 +891,16 @@ close_win(Win, #state{windows=#{frame:=TopFrame,ch:=Tree,loose:=Loose,szr:=Szr}=
 		    State
 	    end;
 	#win{frame=Obj} ->
-	    Close = fun(Where, Other, GrandP) -> close_window(Obj, Where, Other, GrandP, Szr) end,
-	    {ok, Root} = update_win(Obj, Tree, Tree, Close),
-	    check_tree(Root, Tree),
-	    %% wxSizer:layout(Szr),
-            wxFrame:layout(TopFrame),
-            wxWindow:refresh(TopFrame),
+            Close = fun(Where, Other, GrandP) -> close_window(Obj, Where, Other, GrandP, Szr) end,
+            Root = wx:batch(fun() ->
+                                    {ok, Root} = update_win(Obj, Tree, Tree, Close),
+                                    check_tree(Root, Tree),
+                                    wxFrame:layout(TopFrame),
+                                    wxWindow:refresh(TopFrame),
+                                    Root
+                            end),
+            timer:sleep(200), %% give wx time to update windows on X11
+            wings_io:reset_video_mode_for_gl(0,0),
 	    State#state{windows=Wins#{ch:=Root}}
     end.
 
@@ -995,7 +996,9 @@ setup_detach(#win{frame=Container}=Win, #{szr:=Szr, ch:=Child, frame:=Parent}) -
 
 do_detach_window(#win{frame=Container, win=Child, title=Label}=Win,
 		 Split, Other, GrandP, Top, Szr) ->
-    PosSz = wxWindow:getScreenRect(Container),
+    {X,Y,_,_} = wxWindow:getScreenRect(Container),
+    {W,H} = wxWindow:getClientSize(Container),
+    PosSz = {X,Y,W,H},
     wxWindow:setSize(Container, {-1,-1,1,1}),
     wxWindow:reparent(Container, Top),
     FrameW = make_external_win(Top, Child, PosSz, Label),
@@ -1177,10 +1180,11 @@ make_close_button(Parent, Bar, WBSz, H) ->
     wxWindow:connect(SBM, left_up, [{callback, CB}]).
 
 export_loose(Windows) ->
-    Exp = fun(#win{name=Name, frame=Frame}) ->
-		  {X0,Y0,W,H} = wxWindow:getRect(Frame),
+    Exp = fun(#win{name=Name, win=Win, frame=Frame}) ->
+		  {X0,Y0,_,_} = wxWindow:getRect(Frame),
 		  Pos = wxWindow:screenToClient(?GET(top_frame),{X0,Y0}),
-		  {Name, Pos, {W,H}, []}
+		  Size = wxWindow:getClientSize(Win),
+		  {Name, Pos, Size, []}
 	  end,
     [Exp(Win) || Win <- Windows].
 
@@ -1190,7 +1194,7 @@ export_contained(Root) ->
     Path = find_path(geom, Root),
     lists:reverse(tree_to_list(Root, Path, [])).
 
-update_layout([{_Wh,Mode,Permille}|Contained], #split{obj=Obj, mode=Mode, w1=W10, w2=W20}) ->
+update_layout([{SpltMode,Mode,Permille}|Contained], #split{obj=Obj, mode=Mode, w1=W10, w2=W20}) ->
     {W,H} = wxWindow:getClientSize(Obj),
     Pos =
 	case Mode of
@@ -1198,9 +1202,14 @@ update_layout([{_Wh,Mode,Permille}|Contained], #split{obj=Obj, mode=Mode, w1=W10
 	    splitHorizontally -> round(H * Permille / 1000)
 	end,
     wxSplitterWindow:setSashPosition(Obj, Pos),
-    Cont = update_layout(Contained, W10),
-    update_layout(Cont, W20);
-update_layout(_, Cont) -> Cont.
+    {W1,W2} =
+	case SpltMode of
+	    split -> {W10,W20};
+	    split_rev -> {W20,W10}
+	end,
+    Cont = update_layout(Contained, W1),
+    update_layout(Cont, W2);
+update_layout(Contained, _) -> Contained.
 
 tree_to_list(#split{obj=Obj, mode=Mode,w1=W1,w2=W2}, Path0, Acc0) ->
     SashPos = wxSplitterWindow:getSashPosition(Obj),
@@ -1266,12 +1275,7 @@ init_menubar(Frame) ->
     ets:new(wings_menus, [named_table, public, {keypos,2}]),
     put(wm_active, {menubar, geom}),
     MB = wxMenuBar:new(),
-    try
-	wings_menu:setup_menus(MB, top_menus()),
-	wxFrame:setMenuBar(Frame, MB),
-	erase(wm_active),
-	MB
-    catch _ : Reason ->
-	    io:format("CRASH ~p ~p~n",[Reason, erlang:get_stacktrace()]),
-	    error(Reason)
-    end.
+    wings_menu:setup_menus(MB, top_menus()),
+    wxFrame:setMenuBar(Frame, MB),
+    erase(wm_active),
+    MB.
