@@ -18,7 +18,6 @@
 
 -include("wings.hrl").
 
--ifdef(USE_WX).
 -export([init/0, quit/0, version_info/0,
 	 set_cursor/1,hourglass/0,eyedropper/0,
 	 get_mouse_state/0, is_modkey_pressed/1, is_key_pressed/1,
@@ -122,11 +121,21 @@ blank(PreDef) ->
     end.
 
 set_cursor(CursorId) ->
-    Cursors = ?GET(cursors),
-    Cursor = proplists:get_value(CursorId, Cursors),
-    wx_misc:setCursor(Cursor),
-    put(active_cursor, CursorId),
-    ok.
+    case get(active_cursor) of
+        CursorId -> ok;
+        _ ->
+            Cursors = ?GET(cursors),
+            Cursor = proplists:get_value(CursorId, Cursors),
+            case os:type() of
+                {win32, _} ->
+                    Frame = ?GET(top_frame),
+                    wxWindow:setCursor(Frame, Cursor);
+                {_, _} ->
+                    wx_misc:setCursor(Cursor)
+            end,
+            put(active_cursor, CursorId),
+            ok
+    end.
 
 get_mouse_state() ->
     wx:batch(fun() ->
@@ -574,4 +583,3 @@ sdl_key_map(?SDLK_RSUPER) -> ?WXK_WINDOWS_RIGHT;
 
 sdl_key_map(Key) -> Key.
 
--endif.
