@@ -356,7 +356,9 @@ popup_event_handler({click, Id, Click, Ns}, {Parent,Owner}=Own, Entries0) ->
     case popup_result(lists:keyfind(Id, 2, Entries0), Click, Ns, Owner) of
 	pop -> pop;
 	{submenu, Names, Menus, MagnetClick} ->
-	    {_, X, Y} = wings_io:get_mouse_state(),
+	    {_, X0, Y0} = wings_io:get_mouse_state(),
+	    Pos = wxWindow:screenToClient(wings_wm:this_win(), {X0,Y0}),
+	    {X,Y} = wxWindow:clientToScreen(wings_wm:this_win(), Pos),
 	    Entries = wx_popup_menu(Parent, {X,Y}, Names, Menus, MagnetClick, Owner),
 	    {replace, fun(Ev) -> popup_event_handler(Ev, Own, Entries) end}
     end;
@@ -713,6 +715,17 @@ update_menu(Menu, Item, Cmd0, Help) ->
     MI = case ets:lookup(wings_menus, Id) of
 	     [#menu{object=MO}] ->
 		 MO;
+	     [] when Menu =:= file, element(1, Item) =:= load_pref ->
+		 Names0 = build_names(Item, [Menu]),
+		 Names = lists:droplast(Names0),
+		 [#menu{object=CustomTheme, type=submenu}] = ets:match_object(wings_menus, #menu{name=Names, _ = '_'}),
+		 N  = wxMenu:getMenuItemCount(CustomTheme),
+		 MO = wxMenu:insert(CustomTheme, N, ?wxITEM_NORMAL, [{text, Cmd0}]),
+		 ME=#menu{name=build_command(Item,[Menu]), object=MO,
+			  wxid=Id, type=?wxITEM_NORMAL},
+		 true = ets:insert(wings_menus, ME),
+		 wxMenu:insertSeparator(CustomTheme, N),
+		 MO;
 	     [] when Menu =:= file, element(1, Item) =:= recent_file ->
 		 FileId = predefined_item(menu, Menu),
 		 [#menu{object=File, type=submenu}] = ets:lookup(wings_menus, FileId),
