@@ -12,7 +12,7 @@
 %%
 
 -module(wings_vertex_cmd).
--export([menu/3,command/2,tighten/3,tighten/4,
+-export([menu/3,command/2,tighten_vs/2,tighten_vs/3,
 	 connect/2,bevel_vertex/2,flatten/2]).
 
 -export([set_color/2]).
@@ -77,9 +77,9 @@ command({deform,Deform}, St0) ->
 command(auto_smooth, St) ->
     wings_body:auto_smooth(St);
 command(dissolve, St) ->
-    {save_state,wings_shape:update_folders(dissolve(St))};
+    {save_state,dissolve(St)};
 command(collapse, St) ->
-    {save_state,wings_shape:update_folders(wings_collapse:collapse(St))};
+    {save_state,wings_collapse:collapse(St)};
 command({move,Type}, St) ->
     wings_move:setup(Type, St);
 command({rotate,Type}, St) ->
@@ -124,19 +124,18 @@ flatten(Plane0, Center, St) ->
 %%%
 
 extrude(Type, St0) ->
-    {St,Tvs} = wings_sel:mapfold(
-		 fun(Vs, We0, Acc) ->
-			 extrude_vertices(Vs, We0, Acc)
-		 end, [], St0),
-    wings_move:plus_minus(Type, Tvs, St).
+    MF = fun(Vs, We0) -> extrude_vertices(Vs, We0) end,
+    St = wings_sel:map(MF, St0),
+    DF = fun(_, #we{temp=PlusMinus}) -> PlusMinus end,
+    wings_move:plus_minus(Type, DF, St).
 
-extrude_vertices(Vs, We0, Acc) ->
+extrude_vertices(Vs, We0) ->
     We = gb_sets:fold(
 	   fun(V, A) ->
 		   ex_new_vertices(V, We0, A)
 	   end, We0, Vs),
     NewVs = wings_we:new_items_as_ordset(vertex, We0, We),
-    {We,[{Vs,NewVs,gb_sets:empty(),We}|Acc]}.
+    We#we{temp={Vs,NewVs,gb_sets:empty()}}.
 
 ex_new_vertices(V, OrigWe, #we{vp=Vtab}=We0) ->
     Center = wings_vertex:pos(V, We0),
@@ -177,14 +176,18 @@ ex_connect([], _, We) ->
 %%%
 
 bevel(St0) ->
-    {St,{Tvs0,FaceSel}} =
-	wings_sel:mapfold(
-	  fun(VsSet, We, A) ->
-		  bevel_1(VsSet, We, A)
-	  end, {[],[]}, St0),
-    {Min,Tvs} = bevel_normalize(Tvs0),
-    wings_drag:setup(Tvs, [{distance,{0.0,Min}}],
-		     wings_sel:set(face, FaceSel, St)).
+    St = wings_sel:map_update_sel(
+           fun(VsSet, We) ->
+                   bevel_1(VsSet, We)
+           end, face, St0),
+    Limit = bevel_limit(St),
+    DF = fun(_, #we{temp={_,Tv}}) -> Tv end,
+    wings_drag:fold(DF, [{distance,{0.0,Limit}}], St).
+
+bevel_limit(St) ->
+    MF = fun(_, #we{temp={Limit,_}}) -> Limit end,
+    RF = fun min/2,
+    wings_sel:dfold(MF, RF, infinity, St).
 
 bevel_vertex(V, We0) ->
     Es = wings_vertex:fold(
@@ -198,14 +201,14 @@ bevel_vertex(V, We0) ->
 	    We
     end.
 
-bevel_1(VsSet, #we{id=Id}=We0, {Tvs,Fa}) ->
+bevel_1(VsSet, We0) ->
     Vs = gb_sets:to_list(VsSet),
-    {We,Tv,WeTrans,Fs0} = bevel_vertices(Vs, VsSet, We0, We0, [], [], []),
-    FaceSel = case Fs0 of
-		  [] -> Fa;
-		  _ -> [{Id,gb_sets:from_list(Fs0)}|Fa]
-	      end,
-    {We,{[{Id,WeTr} || WeTr <- WeTrans]++[{Id,Tv}|Tvs],FaceSel}}.
+    {We1,VecVs0,WeTrans,Fs0} = bevel_vertices(Vs, VsSet, We0, We0, [], [], []),
+    {VecVs,Limit} = bevel_normalize(VecVs0),
+    Tv = {we,WeTrans,wings_drag:translate_fun(VecVs, We1)},
+    We = We1#we{temp={Limit,Tv}},
+    Fs = gb_sets:from_list(Fs0),
+    {We,Fs}.
 
 bevel_vertices([V|Vs], VsSet, WeOrig, We0, Acc0, Facc, WeTrans0) ->
     Adj = adjacent(V, VsSet, WeOrig),
@@ -249,7 +252,7 @@ bevel_vertex_1(V, Es, NumEdges, Adj, We0, Vec0, WeTrans0) ->
 	    true ->
 		%% Define a fun to update the vertex attributes dynamically.
 		VaTrans = bevel_va_fun(V, Es, InnerFace, We0),
-		WeTrans1 = [{we,VaTrans}|WeTrans0],
+		WeTrans1 = [VaTrans|WeTrans0],
 
 		%% Calculate initial values for vertex attributes.
 		%% This is necessary to get correct results in case two
@@ -326,24 +329,11 @@ bevel_new_vertices(Ids, N, Pos, Vct0, Vtab0) when N > 0 ->
     bevel_new_vertices(wings_we:bump_id(Ids), N-1, Pos, Vct, Vtab);
 bevel_new_vertices(_, _, _, Vct, Vtab) -> {Vct,Vtab}.
 
-bevel_normalize(Tvs) ->
-    bevel_normalize(Tvs, 1.0E207, []).
-
-bevel_normalize([{_,{we,_}}=WeTrans|Tvs], Min, Acc) ->
-    bevel_normalize(Tvs, Min, [WeTrans|Acc]);
-bevel_normalize([{Id,VecVs0}|Tvs], Min0, Acc) ->
-    {VecVs,Min} = bevel_normalize_1(VecVs0, Min0),
-    bevel_normalize(Tvs, Min, [{Id,VecVs}|Acc]);
-bevel_normalize([], Min, Tvs) -> {Min,Tvs}.
-
-bevel_normalize_1(VecVs, Min0) ->
+bevel_normalize(VecVs) ->
     mapfoldl(fun({Vec,V}, M0) ->
-		     Min = case e3d_vec:len(Vec) of
-			       Len when Len < M0 -> Len;
-			       _Len -> M0
-			   end,
+		     Min = min(e3d_vec:len(Vec), M0),
 		     {{e3d_vec:norm(Vec),[V]},Min}
-	     end, Min0, VecVs).
+	     end, infinity, VecVs).
 
 adjacent(V, Vs, We) ->
     wings_vertex:fold(
@@ -440,61 +430,54 @@ connect(Vs0, #we{mirror=MirrorFace}=We) ->
 	  end, We, FaceVs).
 
 connecting_edge(St0) ->
-    {St,Sel} = wings_sel:mapfold(fun connecting_edge/3, [], St0),
-    wings_sel:set(edge, Sel, St).
+    wings_sel:map_update_sel(fun connecting_edge/2, edge, St0).
 
-connecting_edge(Vs0, #we{mirror=MirrorFace, id=Id}=We0, A) ->
+connecting_edge(Vs0, #we{mirror=MirrorFace}=We0) ->
     FaceVs = wings_vertex:per_face(Vs0, We0),
     We1 = lists:foldl(fun({Face,_}, Acc) when Face =:= MirrorFace -> Acc;
 			 ({Face,Vs}, Acc) -> wings_vertex:connect(Face, Vs, Acc)
 		      end, We0, FaceVs),
     Sel = wings_we:new_items_as_gbset(edge, We0, We1),
-    {We1,[{Id,Sel}|A]}.
+    {We1,Sel}.
 
-connect_cuts(#st{}=St0) ->
-    Do = fun(Set, #we{id=Id}=We0, #st{sel=Sel} = St) ->
-		 Sz = gb_sets:size(Set),
-		 case 1 < Sz andalso Sz < 4 of
-		     true -> ok;
-		     false -> connect_cuts_error()
-		 end,
-		 List = gb_sets:to_list(Set),
-		 Dict = combinations(List),
-		 Connect = fun({VS0,VE0}, {Set0,#we{}=We1}) ->
-				   {Set2,We2} = wings_vertex:connect_cut(VS0,VE0,We1),
-				   {gb_sets:union(Set2,Set0), We2}
-			   end,
-		 {Es,We} = lists:foldl(Connect, {gb_sets:empty(), We0}, Dict),
-		 %% Make selection
-		 wings_shape:replace(Id, We, wings_sel:set([{Id,Es}|Sel], St))
-	 end,
-    wings_sel:fold(Do, St0#st{selmode=edge, sel=[]}, St0).
+connect_cuts(St) ->
+    wings_sel:map_update_sel(fun connect_cuts_fun/2, edge, St).
+
+connect_cuts_fun(Vs0, We) ->
+    case gb_sets:size(Vs0) of
+        2 -> ok;
+        3 -> ok;
+        _ -> connect_cuts_error()
+    end,
+    Vs = gb_sets:to_list(Vs0),
+    VsPairs = [{A,B} || A <- Vs, B <- Vs, A < B],
+    do_connect_cuts(VsPairs, gb_sets:empty(), We).
+
+do_connect_cuts([{VS0,VE0}|Pairs], Es0, We0) ->
+    {We,Es1} = wings_vertex:connect_cut(VS0, VE0, We0),
+    Es = gb_sets:union(Es0, Es1),
+    do_connect_cuts(Pairs, Es, We);
+do_connect_cuts([], Es, We) ->
+    {We,Es}.
 
 -spec connect_cuts_error() -> no_return().
 connect_cuts_error() ->
     Msg = ?__(1, "Defined only for two or three selected vertices, per object."),
     wings_u:error_msg(Msg).
 
-combinations(List) ->
-    [{A,B} || A<-List, B<-List, A < B].
 
 %%%
 %%% The Tighten command.
 %%%
 
 tighten(St) ->
-    Tvs = wings_sel:fold(fun tighten/3, [], St),
-    wings_drag:setup(Tvs, [percent], St).
+    wings_drag:fold(fun tighten_vs/2, [percent], St).
 
-tighten(Vs, #we{id=Id}=We, Acc) when is_list(Vs) ->
-    Tv = foldl(
-	   fun(V, A) ->
-		   Vec = tighten_vec(V, We),
-		   [{Vec,[V]}|A]
-	   end, [], Vs),
-    [{Id,Tv}|Acc];
-tighten(Vs, We, Acc) -> 
-    tighten(gb_sets:to_list(Vs), We, Acc).
+tighten_vs(Vs, We) when is_list(Vs) ->
+    Translate = [{tighten_vec(V, We),[V]} || V <- Vs],
+    wings_drag:translate_fun(Translate, We);
+tighten_vs(Vs, We) ->
+    tighten_vs(gb_sets:to_list(Vs), We).
 
 tighten_vec(V, #we{vp=Vtab,mirror=MirrorFace}=We) ->
     Cs = wings_vertex:fold(
@@ -512,27 +495,26 @@ tighten_vec(V, #we{vp=Vtab,mirror=MirrorFace}=We) ->
 %%%
 
 tighten(Magnet, St) ->
-    Tvs = wings_sel:fold(fun(Vs, We, Acc) ->
-				 tighten(Vs, We, Magnet, Acc)
-			 end, [], St),
     Flags = wings_magnet:flags(Magnet, []),
-    wings_drag:setup(Tvs, [percent,falloff], Flags, St).
+    wings_drag:fold(fun(Vs, We) ->
+                            tighten_vs(Vs, We, Magnet)
+                    end, [percent,falloff], Flags, St).
 
-tighten(Vs, We, Magnet, Acc) when is_list(Vs) ->
+tighten_vs(Vs, We, Magnet) when is_list(Vs) ->
     Tv = foldl(
 	   fun(V, A) ->
 		   Vec = tighten_vec(V, We),
 		   [{Vec,[V]}|A]
 	   end, [], Vs),
-    magnet_move(Tv, Magnet, We, Acc);
-tighten(Vs, We, Magnet, Acc) -> 
-    tighten(gb_sets:to_list(Vs), We, Magnet, Acc).
+    magnet_move(Tv, Magnet, We);
+tighten_vs(Vs, We, Magnet) ->
+    tighten_vs(gb_sets:to_list(Vs), We, Magnet).
 
-magnet_move(Tv, Magnet0, #we{id=Id}=We, Acc) ->
+magnet_move(Tv, Magnet0, We) ->
     Vs = lists:append([Vs || {_,Vs} <- Tv]),
     {VsInf,Magnet,Affected} = wings_magnet:setup(Magnet0, Vs, We),
     Vec = magnet_tighten_vec(Affected, We, []),
-    [{Id,{Affected,wings_move:magnet_move_fun(Vec, VsInf, Magnet)}}|Acc].
+    {Affected,wings_move:magnet_move_fun(Vec, VsInf, Magnet)}.
 
 magnet_tighten_vec([V|Vs], We, Acc) ->
     Vec = tighten_vec(V, We),
