@@ -257,6 +257,12 @@ delete_windows(Name, W0) ->
 	none ->
 	    W0;
 	{value,#win{obj=Obj, links=Links}} ->
+            case get(current_gl) of
+                Obj ->
+                    erase(current_gl),
+                    wxGLCanvas:setCurrent(wxwindow(geom));
+                _ -> ignore
+            end,
 	    erase(Obj),
 	    delete_props(Name),
 	    This = this(),
@@ -724,17 +730,25 @@ dispatch_event(#wx{obj=Obj, event=#wxSize{size={W,H}}}) ->
 	false ->
 	    true
     end;
-dispatch_event(#wx{obj=Obj, event=#wxFocus{}}) ->
+dispatch_event(#wx{obj=Obj, event=#wxFocus{type=kill_focus, win=New}}) ->
     case {get_focus_window(), wx2win(Obj)} of
         {undefined, _} -> ok;
         {Win, Win} ->
             update_focus(none);
         {{grabbed, Win}, Win} ->
-            io:format("Grabbed focus lost: ~p ~p~n", [Obj, Win]),
-            do_dispatch(Win, lost_focus),
-            update_focus(none);
+            case wx:equal(New, wxwindow(Win)) of
+                true -> %% For some reason wxWidgets sends focus lost
+                    ok; %% to the already focused window, just ignore
+                false ->
+                    ?dbg("Grabbed focus lost: ~p ~p~n", [Obj, Win]),
+                    do_dispatch(Win, lost_focus),
+                    update_focus(none)
+            end;
         {{grabbed, OtherWin}, Win} ->
-            io:format("Grabbed focus lost old?: ~p ~p~n", [OtherWin, Win]),
+            case OtherWin of
+                dialog_blanket -> ignore;
+                _ -> ?dbg("Grabbed focus lost old?: ~p ~p~n", [OtherWin, Win])
+            end,
             ok;
         {_OldFocus, _CurrentFocus} ->
             ok
@@ -827,14 +841,12 @@ update_focus(Active) ->
 		    Active
 	    end;
 	Win ->
-	    do_dispatch(Active, got_focus),
 	    Win
     end.
 
 do_dispatch(Active, Ev) ->
     case lookup_window_data(Active) of
 	none ->
-	    io:format("~p:~p: Dropped Event ~p: ~p~n",[?MODULE,?LINE, Active, Ev]),
 	    false;
 	Win0 ->
 	    case send_event(Win0, Ev) of
@@ -1016,6 +1028,9 @@ handle_response(Res, Event, Stk0) ->
 	{replace,Top,Continue} when is_function(Top), is_function(Continue) ->
 	    Stk = replace_handler(Top, Stk0),
 	    handle_event(Continue, dummy_event, Stk);
+        {replace,Top,Ev} when is_function(Top) ->
+	    Stk = replace_handler(Top, Stk0),
+	    handle_event(hd(Stk), Ev, Stk);
 	{pop_handler,_}=PopH -> replace_handler(PopH, Stk0);
 	Top when is_function(Top) -> replace_handler(Top, Stk0)
     end.
